/**
 * @file gaim-fifo.h  A FIFO byte queue.
 * @ingroup core
 *
 * gaim
 *
 * Gaim is the legal property of its developers, whose names are too numerous
 * to list here.  Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GAIM_FIFO_H_
#define _GAIM_FIFO_H_

#include <glib.h>

G_BEGIN_DECLS

typedef struct
{
	GList *buffer_list;
	GList *enqueue_buffer;
	gint   enqueue_pos;
	GList *dequeue_buffer;
	gint   dequeue_pos;
}
GaimFifo;

GaimFifo *gaim_fifo_new          (void);
void      gaim_fifo_destroy      (GaimFifo *fifo);

gint      gaim_fifo_get_used     (GaimFifo *fifo);

void      gaim_fifo_enqueue      (GaimFifo *fifo, gconstpointer src, gint n);
gboolean  gaim_fifo_dequeue      (GaimFifo *fifo, gpointer dest, gint n);
gboolean  gaim_fifo_dequeue_line (GaimFifo *fifo, gchar **dest);

gboolean  gaim_fifo_peek         (GaimFifo *fifo, gpointer dest, gint n);

G_END_DECLS

#endif /* _GAIM_FIFO_H_ */
