/**
 * @file gaim-io.h  Buffering I/O layer.
 * @ingroup core
 *
 * gaim
 *
 * Gaim is the legal property of its developers, whose names are too numerous
 * to list here.  Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _GAIM_IO_H_
#define _GAIM_IO_H_

#include "gaim-fifo.h"
#include "sslconn.h"

G_BEGIN_DECLS

typedef struct GaimIO GaimIO;

typedef void (*GaimIOFunc) (GaimIO *io, gpointer data);

struct GaimIO
{
	gint               fd;
	GaimSslConnection *gsc;

	guint              got_eof : 1;

	GaimFifo          *read_fifo;
	GaimFifo          *write_fifo;

	guint              read_watch_id;
	guint              write_watch_id;

	/* Callback invoked when input buffer grows */
	GaimIOFunc         read_func;
	gpointer           read_func_data;

	/* Callback invoked when output buffer shrinks (sufficiently) */
	GaimIOFunc         write_func;
	gpointer           write_func_data;

	/* Callback invoked when connection drops */
	GaimIOFunc         eof_func;
	gpointer           eof_func_data;

	/* Callback invoked when output buffer empty */
	GaimIOFunc         flushed_func;
	gpointer           flushed_func_data;
};

GaimIO            *gaim_io_new                (gint fd);
GaimIO            *gaim_io_new_ssl            (GaimSslConnection *gsc);
void               gaim_io_destroy            (GaimIO *io);

gint               gaim_io_get_fd             (GaimIO *io);
GaimSslConnection *gaim_io_get_ssl_connection (GaimIO *io);
gboolean           gaim_io_is_connected       (GaimIO *io);

void               gaim_io_set_read_func      (GaimIO *io, GaimIOFunc func, gpointer data);
void               gaim_io_set_write_func     (GaimIO *io, GaimIOFunc func, gpointer data);
void               gaim_io_set_eof_func       (GaimIO *io, GaimIOFunc func, gpointer data);
void               gaim_io_set_flushed_func   (GaimIO *io, GaimIOFunc func, gpointer data);

gboolean           gaim_io_write_bytes        (GaimIO *io, gconstpointer src, gint n_bytes);
gboolean           gaim_io_write_line         (GaimIO *io, const gchar *line);

gboolean           gaim_io_read_bytes         (GaimIO *io, gpointer dest, gint n_bytes);
gboolean           gaim_io_read_line          (GaimIO *io, gchar **line);

G_END_DECLS

#endif /* _GAIM_IO_H_ */
